package com.abi.qms.platform.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.json.JSONUtil;
import com.abi.base.foundation.util.RedisClient;
import com.abi.qms.platform.dao.entity.*;
import com.abi.qms.platform.dao.enums.*;
import com.abi.qms.platform.dao.mapper.*;
import com.abi.qms.platform.dao.tablestore.entity.QrCode;
import com.abi.qms.platform.dao.vo.result.*;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.constant.RedisKey;
import com.abi.qms.platform.infrastructure.mq.GenerateCodeConsumer;
import com.abi.qms.platform.infrastructure.util.*;
import com.abi.qms.platform.service.FactoryService;
import com.abi.qms.platform.service.MaterialService;
import com.abi.qms.platform.service.QrPackageBookingOrderService;
import com.abi.qms.platform.service.QrPackageService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.abi.task.common.tablestore.TableStorePlusUtils;
import com.abi.task.common.tablestore.TableStoreUtils;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import freemarker.template.Template;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.bouncycastle.util.Pack;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;

import javax.servlet.http.HttpServletResponse;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 码包 Service业务层处理
 *
 * @author WeiganCai
 * @date 2021-04-28
 */
@Service
@Slf4j
public class QrPackageServiceImpl implements QrPackageService {

    @Autowired
    private QrPackageMapper qrPackageMapper;

    @Autowired
    private BaseFactoryMapper baseFactoryMapper;

    @Autowired
    private QrPackageDownloadRecordMapper qrPackageDownloadRecordMapper;

    @Autowired
    private QrPackageBatchMapper qrPackageBatchMapper;

    @Autowired
    private AmqpTemplate amqpTemplate;

    @Autowired
    private SendmailUtil sendmailUtil;

    @Autowired
    private SmsUtil smsUtil;

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private UserUtil userUtil;

    @Autowired
    private TableStorePlusUtils tableStorePlusUtils;

    @Autowired
    private QrRepertoryColumnMapper repertoryColumnMapper;

    @Autowired
    private BaseActiveMapper baseActiveMapper;

    @Autowired
    private QrBoxMappingMapper qrBoxMappingMapper;

    @Autowired
    private TableStoreUtils tableStoreUtils;

    @Autowired
    private BaseMaterialMapper baseMaterialMapper;

    @Autowired
    private BaseBrandMapper baseBrandMapper;

    @Autowired
    private QrPackageSapOrderItemMapper qrPackageSapOrderItemMapper;

    @Autowired
    private MaterialService materialService;

    @Autowired
    private FactoryService factoryService;

    @Autowired
    private QrBoxCodeFormatMapper qrBoxCodeFormatMapper;

    @Autowired
    private QrPackageBookingOrderService qrPackageBookingOrderService;

    /**
     * 保存码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveQrPackage(SaveQrPackageReq req) {
        boolean isAdd = (req.getId() == null);

        //0-校验入参
        QrPackage qrPackage = checkRequestWhenSaveQrPackage(req);

        //1-自动补全or处理入参数据
        fullParam(req);

        //2-构造码包主表对象
        qrPackage = buildQrPackage(req, qrPackage);

        //3-新增or修改 码包主表
        if (!isAdd) {
            qrPackageMapper.updateById(qrPackage);
        } else {
            qrPackageMapper.insert(qrPackage);
        }

        //4-保存码包批次子表信息
        saveQrPackageBatch(qrPackage, isAdd, req);
    }

    /**
     * 自动补全or处理入参数据
     */
    private void fullParam(SaveQrPackageReq req) {
        //1-如果没有传入二级供应商，则表示选的是无等级（即自己做自己）的包材厂，那么就把自己放入二级供应商内
        if (req.getFactoryCoverLevelTwoId() == null) {
            req.setFactoryCoverLevelTwoId(req.getFactoryCoverId());
        }

        //2-如果批次信息为空（盖码 or 真实业务没有批次的箱码包材厂），则补充批次信息
        if (CollectionUtil.isEmpty(req.getPackageBatchBeanList())) {
            //列表
            List<SaveQrPackageReq.PackageBatchBean> batchBeanList = new ArrayList<>();

            //码的总量
            Long qrNumber = req.getQrNumber();

            //批次信息按包材厂的最大文件数来拆分
            BaseFactory factory = baseFactoryMapper.selectById(req.getFactoryCoverLevelTwoId());
            Long maxFileCount = factory.getMaxFileCount();
            //如果二级包材厂的最大数为空，则认为和总数一致
            if (maxFileCount == null || maxFileCount <= 0) {
                maxFileCount = qrNumber;
            }

            //计算需要生成多少批次，不是整除则+1
            int batchCount = qrNumber.intValue() / maxFileCount.intValue();
            if (batchCount * maxFileCount < qrNumber) {
                batchCount = batchCount + 1;
            }

            //循环添加批次
            for (int i = 0; i < batchCount; i++) {
                //每个批次多少条
                Long batchQrNumber = maxFileCount;
                if (i == batchCount - 1) {
                    batchQrNumber = qrNumber - (batchCount - 1) * maxFileCount;
                }
                SaveQrPackageReq.PackageBatchBean batchBean = new SaveQrPackageReq.PackageBatchBean();
                batchBean.setBatchNumber("000001");
                batchBean.setBatchQrNumber(batchQrNumber);
                batchBeanList.add(batchBean);
            }
            req.setPackageBatchBeanList(batchBeanList);
        }

        //3-SAP:根据物料ID查询物料类型，判断码类型，保存至码包表中
        //预订单:直接预订单放入
        if(CodePackageOrderTypeEnum.SAP_ORDER.is(req.getOrderType())){
            BaseMaterial material = baseMaterialMapper.selectById(req.getMaterialId());
            AssertUtil.isNull(material, "物料信息不存在");
            req.setQrType(getPackageCode(material.getMaterialType()));
        }else{
            QrPackageBookingOrder bookingOrder = qrPackageBookingOrderService.getBookingOrder(req.getBookingOrder());
            AssertUtil.isNull(bookingOrder, "预订单不存在");
            req.setQrType(bookingOrder.getQrType());
        }

        //4-如果是盖码，那么查询盖码得默认箱码幅面，并放入入参
        if(QrTypeEnum.CAP.is(req.getQrType())){
            QueryWrapper<QrBoxCodeFormat> formatQw = new QueryWrapper<>();
            formatQw.eq("is_sys", 1);
            //1-盖码默认幅面
            formatQw.eq("sys_type", 1);
            formatQw.eq("is_delete", 0);
            List<QrBoxCodeFormat> beerFormatList = qrBoxCodeFormatMapper.selectList(formatQw);
            if (CollectionUtil.isEmpty(beerFormatList)) {
                throw new BusinessException("盖码默认幅面与码格式不存在，请检查初始数据");
            }
            QrBoxCodeFormat beerFormat = beerFormatList.get(0);
            req.setBoxCodeFormatId(beerFormat.getId());
        }

    }

    /**
     * 保存码包批次子表信息
     *
     * @param qrPackage
     * @param isAdd
     * @param req
     */
    private void saveQrPackageBatch(QrPackage qrPackage, boolean isAdd, SaveQrPackageReq req) {
        //删除批次号（仅更新码包可能删除）
        if (!isAdd) {
            //查询原有批次号
            QueryWrapper<QrPackageBatch> qrPackageBatchQuery = new QueryWrapper<>();
            qrPackageBatchQuery.eq("package_id", qrPackage.getId());
            qrPackageBatchQuery.eq("is_delete", 0);
            List<QrPackageBatch> qrPackageBatches = qrPackageBatchMapper.selectList(qrPackageBatchQuery);
            List<Long> batchOldIds = qrPackageBatches.stream().map(QrPackageBatch::getId).collect(Collectors.toList());
            //更新的批次数据
            List<Long> batchNewIds = req.getPackageBatchBeanList().stream().map(SaveQrPackageReq.PackageBatchBean::getId).collect(Collectors.toList());
            //原有-入参更新 = 删除的数据
            List<Long> deleteBatch = new ArrayList<>(batchOldIds);
            deleteBatch.removeAll(batchNewIds);
            if (CollectionUtil.isNotEmpty(deleteBatch)) {
                UpdateWrapper<QrPackageBatch> qrPackageBatchDeleteParam = new UpdateWrapper<>();
                qrPackageBatchDeleteParam.in("id", deleteBatch)
                        .set("is_delete", 1);
                qrPackageBatchMapper.update(null, qrPackageBatchDeleteParam);
            }
        }

        //添加或更新
        for (SaveQrPackageReq.PackageBatchBean packageBatchBean : req.getPackageBatchBeanList()) {
            if (packageBatchBean.getId() == null) {
                QrPackageBatch qrPackageBatch = new QrPackageBatch();
                qrPackageBatch.setPackageId(qrPackage.getId());
                qrPackageBatch.setBatchNumber(packageBatchBean.getBatchNumber());
                qrPackageBatch.setBatchQrNumber(packageBatchBean.getBatchQrNumber());
                qrPackageBatchMapper.insert(qrPackageBatch);
            } else {
                QueryWrapper<QrPackageBatch> qrPackageBatchQ = new QueryWrapper<>();
                qrPackageBatchQ.eq("id", packageBatchBean.getId());
                qrPackageBatchQ.eq("is_delete", 0);
                QrPackageBatch qrPackageBatch = qrPackageBatchMapper.selectOne(qrPackageBatchQ);
                if (Objects.isNull(qrPackageBatch)) {
                    throw new BusinessException(ErrorCodeEnum.NO_EXISTS_DATA.getCode(), "批次对象不存在");
                }
                qrPackageBatch.setBatchQrNumber(packageBatchBean.getBatchQrNumber());
                qrPackageBatch.setBatchNumber(packageBatchBean.getBatchNumber());
                qrPackageBatchMapper.updateById(qrPackageBatch);
            }
        }
    }


    /**
     * build数据库的码包对象
     *
     * @param req
     * @param qrPackage
     * @return
     */
    private QrPackage buildQrPackage(SaveQrPackageReq req, QrPackage qrPackage) {
        if (qrPackage == null) {
            qrPackage = new QrPackage();
        }

        qrPackage.setSapOrderNo(req.getSapOrderNo());
        qrPackage.setBookingOrder(req.getBookingOrder());
        qrPackage.setMaterialId(req.getMaterialId());
        qrPackage.setMaterialCode(req.getMaterialCode());
        qrPackage.setFactoryCoverId(req.getFactoryCoverId());
        qrPackage.setFactoryCoverLevelTwoId(req.getFactoryCoverLevelTwoId());
        qrPackage.setFactoryBeerId(req.getFactoryBeerId());
        qrPackage.setBoxCodeFormatId(req.getBoxCodeFormatId());
        qrPackage.setQrNumber(req.getQrNumber());
        qrPackage.setStandbyRatio(req.getStandbyRatio());
        qrPackage.setReasonApply(req.getReasonApply());
        qrPackage.setInvalid(InvalidEnum.NOT_INVALID.getCode());
        qrPackage.setQrType(req.getQrType());

        return qrPackage;
    }

    /**
     * 校验保存码包的入参
     *
     * @param req
     * @return QrPackage 如果是修改码包就把码包查出来
     */
    private QrPackage checkRequestWhenSaveQrPackage(SaveQrPackageReq req) {
        QrPackage qrPackage = null;
        Long qrPackageId = req.getId();
        boolean isAdd = (req.getId() == null);

        // 修改码包时的条件限制
        if (qrPackageId != null) {
            qrPackage = qrPackageMapper.selectById(qrPackageId);
            AssertUtil.isNull(qrPackage, "码包不存在");
            // 只有待审核、审核拒绝状态下才可编辑
            Integer applyStatus = qrPackage.getApplyStatus();
            if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
                throw new BusinessException("码包不可编辑");
            }
        }

        //根据类型判断订单号不能为空
        if (CodePackageOrderTypeEnum.SAP_ORDER.is(req.getOrderType()) && StringUtils.isBlank(req.getSapOrderNo())) {
            throw new BusinessException("SAP订单号不能为空");
        }
        if (CodePackageOrderTypeEnum.BOOKING_ORDER.is(req.getOrderType()) && StringUtils.isBlank(req.getBookingOrder())) {
            throw new BusinessException("预订单号不能为空");
        }

        //如果SAP订单号和物料有重复数据，需提交额外申请原因
        if (StringUtils.isNotBlank(req.getSapOrderNo()) && req.getReasonApply() == null) {
            ValidSAPOrderNoAndMaterialReq reqValid = new ValidSAPOrderNoAndMaterialReq();
            reqValid.setMaterialId(req.getMaterialId());
            reqValid.setSapOrderNo(req.getSapOrderNo());
            reqValid.setApplyNumber(req.getQrNumber());
            ValidSAPOrderNoAndMaterialRes validSAPOrderNoAndMaterialRes = validSAPOrderNoAndMaterial(reqValid);
            if (TrueFalseEnum.TRUE.is(validSAPOrderNoAndMaterialRes.getIsOverstepNum())) {
                throw new BusinessException("该SAP订单号和物料已超额申请码包，若仍需申请需要提交额外码包申请原因");
            }
        }

        //是否箱码
        boolean isCarton = false;
        if(CodePackageOrderTypeEnum.SAP_ORDER.is(req.getOrderType())){
            //查询sap订单，物料信息必填
            BaseMaterial material = baseMaterialMapper.selectById(req.getMaterialId());
            AssertUtil.isNull(material, "物料信息不存在");
            isCarton = QrTypeEnum.CARTON.is(getPackageCode(material.getMaterialType()));
        }else{
            //查询预订单
            QrPackageBookingOrder bookingOrder = qrPackageBookingOrderService.getBookingOrder(req.getBookingOrder());
            AssertUtil.isNull(bookingOrder, "预订单不存在");
            isCarton = QrTypeEnum.CARTON.is(bookingOrder.getQrType());
        }
        //如果是箱码则必传箱码幅面
        if (isCarton && req.getBoxCodeFormatId() == null) {
            throw new BusinessException("箱码幅面为空");
        }

        //若当前登录用户为包材厂用户，则：
        UserInfo wholeUser = userUtil.getWholeUser();
        BaseFactory factoryCover = baseFactoryMapper.selectById(req.getFactoryCoverId());
        //如果选择的是一级包材厂（而不是无等级包材厂），那么二级包材厂必填
        if (FirstFactoryEnum.YES.is(factoryCover.getIsFirstFactory())
                && req.getFactoryCoverLevelTwoId() == null) {
            throw new BusinessException("选择了一级包材厂，则二级包材厂必填");
        }
        if (wholeUser.getFactoryId() != null && FactoryTypeEnum.COVER.is(wholeUser.getFactoryType())) {
            //1.他是包材厂，包材厂必须为该用户所在的包材厂，不可修改
            if (FactoryTypeEnum.COVER.is(factoryCover.getFactoryType())
                    && !req.getFactoryCoverId().equals(wholeUser.getFactoryId())) {
                throw new BusinessException("包材厂只能选择登录用户的包材厂");
            }
            //2.他是二级包材厂，二级包材厂必须为用户所在的包材厂，不可修改
            if (FactoryTypeEnum.TWO_COVER.is(factoryCover.getFactoryType())
                    && !req.getFactoryCoverLevelTwoId().equals(wholeUser.getFactoryId())) {
                throw new BusinessException("二级包材厂只能选择登录用户的包材厂");
            }
        }

        if (!isAdd && wholeUser.getFactoryId() != null) {
            //一级包材厂不允许修改码包
            if (FirstFactoryEnum.YES.is(factoryCover.getIsFirstFactory())) {
                throw new BusinessException("一级包材厂不允许修改码包");
            }
            //二级包材厂or无级包材厂，只能改自己得码包
            if (FirstFactoryEnum.NOT.is(factoryCover.getIsFirstFactory())
                    && !wholeUser.getFactoryId().equals(qrPackage.getFactoryCoverLevelTwoId())) {
                throw new BusinessException("包材厂只允许修改自己的码包");
            }
        }

        //校验批次号
        if (CollectionUtil.isNotEmpty(req.getPackageBatchBeanList())) {
            List<SaveQrPackageReq.PackageBatchBean> packageBatchBeanList = req.getPackageBatchBeanList();
            //计算码包数量总和
            long sum = packageBatchBeanList.stream().mapToLong(SaveQrPackageReq.PackageBatchBean::getBatchQrNumber).sum();
            if (sum != req.getQrNumber()) {
                throw new BusinessException("批次码数量与申请码总量不匹配，请确认后重试");
            }
        }

        return qrPackage;
    }


    /**
     * 码包分页查询
     */
    @Override
    public ListQrPackageRes listQrPackage(ListQrPackageReq req) {
        //数据权限隔离，如果登录账号是包材厂，只能看账号包材厂的数据。否则看全部
        //todo:测试定位问题
        log.info("===码包分页查询数据隔离问题定位==》userUtil.getWholeUser()：" + JSONUtil.toJsonStr(userUtil.getWholeUser()));
        if (userUtil.getWholeUser().getFactoryId() != null && userUtil.getWholeUser().getFactoryType() != null && FactoryTypeEnum.COVER.is(userUtil.getWholeUser().getFactoryType())) {
            req.setFactoryCoverId(userUtil.getWholeUser().getFactoryId());
        }

        // 分页查询
        IPage<QrPackageVO> iPage = qrPackageMapper.listQrPackage(PageUtil.createPage(req), req);
        List<QrPackageVO> qrPackageList = iPage.getRecords();

        // 封装出参、放入分页信息
        ListQrPackageRes res = new ListQrPackageRes();
        PageUtil.copyPageInfo(res, iPage);
        List<ListQrPackageRes.QrPackageBean> qrPackageBeanList = PojoConverterUtils.copyList(qrPackageList, ListQrPackageRes.QrPackageBean.class);
        res.setQrRepertoryBeanList(qrPackageBeanList);

        return res;
    }

    /**
     * 码包查询详情
     */
    @Override
    public GetQrPackageDetailRes getQrPackageDetail(GetQrPackageDetailReq req) {
        // 查询码包对象
        QrPackageVO qrPackageVO = qrPackageMapper.selectQrPackageDetailById(req.getId());
        AssertUtil.isNull(qrPackageVO, "码包不存在");

        //拼接额外审核原因
        String reasonDescForAudit = getReasonDescForAudit(qrPackageVO.getReasonApply(), qrPackageVO.getSapOrderNo(), qrPackageVO.getMaterialCode(),qrPackageVO.getId());
        qrPackageVO.setReasonDescForAudit(reasonDescForAudit);

        // 查询下载列表
        List<QrPackageDownloadRecordVO> qrPackageDownloadRecordList = qrPackageMapper.listQrPackageDownloadRecord(req.getId());
        qrPackageVO.setQrPackageDownloadRecordList(qrPackageDownloadRecordList);

        // 查询批次列表
        List<QrPackageBatchVO> qrPackageBatchVOList = qrPackageMapper.listQrBatch(req.getId());
        qrPackageVO.setQrPackageBatchVOList(qrPackageBatchVOList);

        //转化出参
        GetQrPackageDetailRes res = PojoConverterUtils.copy(qrPackageVO, GetQrPackageDetailRes.class);

        return res;
    }

    /**
     * 如果有额外审核原因，则拼接一个备注用于审核时候查看
     * @param reasonApply
     * @param sapOrderNo
     * @param materialCode
     * @param id
     * @return
     */
    private String getReasonDescForAudit(String reasonApply,String sapOrderNo,String materialCode,Long id){
        //如果没有申请原因，也不用拼接额外的描述
        if(StringUtils.isBlank(reasonApply) || StringUtils.isBlank(sapOrderNo) || StringUtils.isBlank(materialCode)){
            return null;
        }

        //查询是哪些码包使用了这个sap订单号和物料
        QueryWrapper<QrPackage> qrPackageQw = new QueryWrapper<>();
        qrPackageQw.eq("sap_order_no",sapOrderNo);
        qrPackageQw.eq("material_code",materialCode);
        qrPackageQw.ne("id",id);
        qrPackageQw.eq("is_delete",0);
        List<QrPackage> qrPackageList = qrPackageMapper.selectList(qrPackageQw);
        if(CollectionUtil.isEmpty(qrPackageList)){
            return null;
        }

        //聚合拼接描述
        long totalQrNumber = qrPackageList.stream().mapToLong(QrPackage::getQrNumber).sum();
        String packageIds = qrPackageList.stream().map(qrPackage -> String.valueOf(qrPackage.getId())).collect(Collectors.joining(","));

        String desc = String.format("该SAP订单号和物料已申请码包%s,码数量%d", packageIds, totalQrNumber);
        log.info("getReasonDescForAudit result => "+desc);

        return desc;
    }

    /**
     * 活动详情封装出参，显示品牌名称
     *
     * @param res
     * @param packageActive
     */

    private void getBrandCodeName(GetQrPackageDetailRes res, PackageActiveVO packageActive) {

//        res.setActiveCode(packageActive.getActiveCode());
//        res.setActiveName(packageActive.getActiveName());
//        res.setActiveUrl(packageActive.getActiveUrl());
//        res.setActiveType(packageActive.getActiveType());
//        res.setBeginTime(packageActive.getBeginTime());
//        res.setEndTime(packageActive.getEndTime());
//        //多品牌名称
//        StringBuffer brandNames = new StringBuffer();
//        //查询品牌一级列表
//        QueryWrapper<BaseBrand> baseBrandQW = new QueryWrapper<>();
//        baseBrandQW.eq("brand_level",1);
//        List<BaseBrand> baseBrands = baseBrandMapper.selectList(baseBrandQW);
//        Map<String, String> brandCodeMap = baseBrands.stream().collect(Collectors.toMap(BaseBrand::getBrandCode, BaseBrand::getBrandName));
//
//        //品牌名称获取
//        String[] brandCodeArray = packageActive.getBrandCode().split(",");
//        for (String brandCode : brandCodeArray) {
//            String codeName = brandCodeMap.get(brandCode);
//            if (StringUtils.isNotBlank(codeName)) {
//                brandNames.append(codeName).append(",");
//            }
//        }
//
//        if (brandNames.length()>0){
//            brandNames.deleteCharAt(brandNames.length()-1);
//            res.setBrandNames(brandNames.toString());
//        }
    }

    /**
     * 删除码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteQrPackage(DeleteQrPackageReq req) {
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");
        // 只有待审核、审核拒绝状态下才可删除
        Integer applyStatus = qrPackage.getApplyStatus();
        if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
            throw new BusinessException("码包不可删除");
        }

        // 逻辑删除
        QrPackage update = new QrPackage().setId(req.getId()).setIsDelete(1);
        qrPackageMapper.updateById(update);
    }

    /**
     * 审核通过码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void passQrPackage(PassQrPackageReq req) {
        QrPackageVO qrPackage = qrPackageMapper.selectQrPackageDetailById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");

        // 只有待审核、审核拒绝状态下才可通过
        Integer applyStatus = qrPackage.getApplyStatus();
        if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
            throw new BusinessException("码包不可通过");
        }

        QrPackage update = new QrPackage().setId(req.getId()).setApplyStatus(QrPackageApplyStatusEnum.REVIEW_PASS.getCode());
        qrPackageMapper.updateById(update);

        // 码库类型为系统生成时，放入rabbitmq，生成码
        if (QrRepertoryTypeEnum.SYSTEM_GENERATE.is(qrPackage.getQrRepertoryType())) {
            amqpTemplate.convertAndSend(GenerateCodeConsumer.GENERATE_CODE_BY_REPERTORY_QUEUE, req.getId());
        }
    }

    /**
     * 审核拒绝码包
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void refuseQrPackage(RefuseQrPackageReq req) {
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");

        // 只有待审核、审核拒绝状态下才可拒绝
        Integer applyStatus = qrPackage.getApplyStatus();
        if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
            throw new BusinessException("码包不可拒绝");
        }

        QrPackage update = new QrPackage().setId(req.getId()).setApplyStatus(QrPackageApplyStatusEnum.REVIEW_REFUSE.getCode());
        qrPackageMapper.updateById(update);
    }

    @Override
    public void downloadQrPackage(DownloadQrPackageReq req, HttpServletResponse response) {
        //查询码包
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");
        //查询包材厂
        BaseFactory factory = baseFactoryMapper.selectById(qrPackage.getFactoryCoverId());
        AssertUtil.isNull(factory, "包材厂不存在");
        if (StringUtils.isBlank(factory.getKeymanMobile())) {
            throw new BusinessException("包材厂手机号不存在");
        }

        // 只有已生成状态下才可下载
        Integer generateStatus = qrPackage.getGenerateStatus();
        if (QrPackageGenerateStatusEnum.UN_GENERATE.is(generateStatus)) {
            throw new BusinessException("码包未生成不可下载");
        }

        if (QrPackageGenerateStatusEnum.GENERATING.is(generateStatus)) {
            throw new BusinessException("码包生成中不可下载");
        }

        if (QrPackageGenerateStatusEnum.GENERATE_FAIL.is(generateStatus)) {
            throw new BusinessException("码包生成失败不可下载");
        }

        //特殊：校验验证码是否正确，正确才可以下载
        boolean verifyCaptchaResult = verifyCaptcha(factory.getKeymanMobile(), req.getCaptcha(), qrPackage.getId());
        if (!verifyCaptchaResult) {
            throw new BusinessException("验证码错误");
        }

        //开始下载
        String url = qrPackage.getDownloadPath();
        try {
            new OssFileDownloadUtil().downloadFile(url, response);
        } catch (Exception e) {
            log.error("下载失败", e);
            throw new BusinessException("下载失败");
        }

        //插入码包下载记录
        Long userId = userUtil.getUser().getId();
        QrPackageDownloadRecord downloadRecord = new QrPackageDownloadRecord()
                .setCreateBy(userId)
                .setQrPackageId(qrPackage.getId());
        qrPackageDownloadRecordMapper.insert(downloadRecord);
        //更新码包表下载时间
        QrPackage qrPackages = new QrPackage()
                .setDownloadTime(LocalDateTime.now());
        qrPackageMapper.updateById(qrPackages);
    }

    /**
     * 码生成失败回滚
     */
    @Override
    public void failedRollback(QrPackage qrPackage) {
        // 更改生成状态为生成失败
        QrPackage qrPackageUpdate = new QrPackage().setId(qrPackage.getId()).setGenerateStatus(QrPackageGenerateStatusEnum.GENERATE_FAIL.getCode());
        qrPackageMapper.updateById(qrPackageUpdate);
    }

    /**
     * 校验是否存在SAP订单号和物料重复数据
     *
     * @param req
     * @return
     */
    @Override
    public ValidSAPOrderNoAndMaterialRes validSAPOrderNoAndMaterial(ValidSAPOrderNoAndMaterialReq req) {
        //查询存在的列表
        QueryWrapper<QrPackage> qrPackageQuery = new QueryWrapper<>();
        qrPackageQuery.eq("material_id", req.getMaterialId());
        qrPackageQuery.eq("sap_order_no", req.getSapOrderNo());
        qrPackageQuery.eq("is_delete", 0);
        List<QrPackage> qrPackages = qrPackageMapper.selectList(qrPackageQuery);
        long num = 0;
        //根据物料id,sap订单编号，计算历史订单总数
        if (CollectionUtil.isNotEmpty(qrPackages)) {
            num = qrPackages.stream().mapToInt(x -> Math.toIntExact(x.getQrNumber())).sum();
        }
        //根据物料，获取sap物料code
        BaseMaterial baseMaterial = baseMaterialMapper.selectById(req.getMaterialId());
        AssertUtil.isNull(baseMaterial, "物料信息不存在");
        //入参获取sap订单表中采购数量
        QueryWrapper<QrPackageSapOrderItem> item = new QueryWrapper<>();
        item.eq("sap_order_no", req.getSapOrderNo());
        item.eq("sap_material_id", baseMaterial.getMaterialCode());
        QrPackageSapOrderItem qrPackageSapOrderItem = qrPackageSapOrderItemMapper.selectOne(item);
        AssertUtil.isNull(qrPackageSapOrderItem, "sap订单不存在");
        //之前采购数量+当前申请数量
        long number = num + req.getApplyNumber();
        //构造出参
        ValidSAPOrderNoAndMaterialRes res = new ValidSAPOrderNoAndMaterialRes();
        //判断是否超出sap订单中的采购数量
        if (number > qrPackageSapOrderItem.getPurchaseCount()) {
            res.setIsOverstepNum(TrueFalseEnum.TRUE.getCode());
        } else {
            res.setIsOverstepNum(TrueFalseEnum.FALSE.getCode());
        }
        return res;
    }

    /**
     * 码包生成完成后发送邮件给码包厂负责人邮箱
     *
     * @param targetEmail
     * @param sapOrderNo
     * @param filename
     * @param password
     */
    @Override
    public void sendEmailForQrPackageCreated(String targetEmail, String sapOrderNo, String filename, String password) {
        try {
            //拿到freemarker模板
            Template template = FreeMarkerUtils.getTemplate("factory.ftl");

            //填充的内容
            Map paramMap = new HashMap<>();
            paramMap.put("sapOrderNo", sapOrderNo);
            paramMap.put("filename", filename);
            paramMap.put("password", password);

            //生成需要发送的内容
            String emailContent = FreeMarkerTemplateUtils.processTemplateIntoString(template, paramMap);

            sendmailUtil.doSendHtmlEmail("QMS系统通知", emailContent, targetEmail);

        } catch (Exception e) {
            log.info("发送码包通知邮件异常", e);
            throw new BusinessException("发送码包通知邮件异常");
        }
    }

    /**
     * 发送短信验证码
     *
     * @param req
     */
    @Override
    public void sendMobileMsg(SendMobileMsgReq req) {
        //查询码包
        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "码包不存在");
        //查询包材厂
        BaseFactory factory = baseFactoryMapper.selectById(qrPackage.getFactoryCoverId());
        AssertUtil.isNull(factory, "包材厂不存在");
        if (StringUtils.isBlank(factory.getKeymanMobile())) {
            throw new BusinessException("包材厂手机号不存在");
        }

        //key:packId:mobile
        Long packId = qrPackage.getId();
        String mobile = factory.getKeymanMobile();
        String key = MessageFormat.format("{0}{1}:{2}", RedisKey.DOWNLOAD_MOBILE_MSG, packId, mobile);

        //获取随机验证码
        String captcha = RandomCodeUtils.getCaptcha(6);

        //发送短信
        smsUtil.sendCode(mobile, captcha);

        //将验证码存入redis
        redisClient.getRedisTemplate().opsForValue().set(key, captcha, 30 * 60, TimeUnit.SECONDS);
    }

    /**
     * 校验校验码
     *
     * @param mobile
     * @param inputCaptcha
     * @param packId
     * @return
     */
    private boolean verifyCaptcha(String mobile, String inputCaptcha, Long packId) {
        String key = MessageFormat.format("{0}{1}:{2}", RedisKey.DOWNLOAD_MOBILE_MSG, packId, mobile);
        log.info("验证下载key:{}", key);
        String captcha = redisClient.get(key);
        log.info(inputCaptcha + "--------------------" + captcha);

        //不一致返回false
        if (!inputCaptcha.equals(captcha)) {
            return false;
        }

        //删除校验码
        redisClient.delete(key);

        return true;
    }

    @Override
    public GetQrSingleCheckDetailRes getQrSingleCheckDetail(GetQrSingleCheckDetailCodeReq req) {

        //调用tablestore查询码信息
        QrCode qrCode = tableStorePlusUtils.selectOne(QrCode.class, req.getQrCode());
        AssertUtil.isNull(qrCode.getPackageId(), "码数据不存在");

        //1-查询码关联信息
        QrSingleCheckVO qrSingleCheckVO = qrPackageMapper.getQrSingleCheckDetailById(Long.valueOf(qrCode.getPackageId()));
        AssertUtil.isNull(qrSingleCheckVO, "码数据不存在");

        //查询是否存在隐形码
        QrRepertoryColumn qrRepertoryColumn = repertoryColumnMapper.selectById(qrCode.getQrRepertoryColumnId());

        //存在别名（隐形码）
        if (!ObjectUtils.isEmpty(qrRepertoryColumn)) {
            qrSingleCheckVO.setAlias(qrRepertoryColumn.getAlias());//隐形码
        }

        //查询码包对应活动单条数据
        if(null != qrSingleCheckVO.getActiveId()) {
            //存在活动数据
            PackageActiveVO packageActive = baseActiveMapper.getPackageActive(qrSingleCheckVO.getActiveId());
            if (!ObjectUtils.isEmpty(packageActive)) {
                qrSingleCheckVO.setActiveName(packageActive.getActiveName());
                qrSingleCheckVO.setActiveUrl(packageActive.getActiveUrl());
            }
        }

        //查询批次号id
        QrOnePackageBatchReq qrOnePackageBatchReq = new QrOnePackageBatchReq();
        qrOnePackageBatchReq.setPackageId(qrCode.getPackageId());
        qrOnePackageBatchReq.setBatchNumber(qrCode.getBatchNumberId());
        QrPackageBatchVO qrPackageBatchOne = qrPackageBatchMapper.getQrPackageBatchOne(qrOnePackageBatchReq);

        //存批次号
        if (ObjectUtils.isNotEmpty(qrPackageBatchOne)) {
            qrSingleCheckVO.setBatchNumber(qrPackageBatchOne.getBatchNumber());
        }

        //1-查询码关联激活信息单条数据
        QrOneBoxCodeMappingReq qrOneBoxCodeMappingReq = new QrOneBoxCodeMappingReq();
        qrOneBoxCodeMappingReq.setPackageId(qrCode.getPackageId());
        qrOneBoxCodeMappingReq.setQrIndex(qrCode.getCodeIndex());
        QrBoxMappingVO qrBoxMappingVO = qrBoxMappingMapper.getIndexQrBoxCodeMappingOne(qrOneBoxCodeMappingReq);

        //存在激活关联信息
        if (ObjectUtils.isNotEmpty(qrBoxMappingVO)) {
            qrSingleCheckVO.setBoxCode(qrBoxMappingVO.getBoxCode());//条码序号
            qrSingleCheckVO.setActiveTime(qrBoxMappingVO.getActiveTime());//激活时间
            qrSingleCheckVO.setActiveUserName(qrBoxMappingVO.getActiveUserName());//激活时间
            qrSingleCheckVO.setActiveStatus(qrBoxMappingVO.getActiveStatus());//激活状态
        }

        //转化出参
        GetQrSingleCheckDetailRes res = PojoConverterUtils.copy(qrSingleCheckVO, GetQrSingleCheckDetailRes.class);
        res.setQrCode(qrCode.getCode());
        return res;
    }

    @Override
    public void invalidQrPackage(InvalidQrPackageReq req) {

        //获取登录用户信息
        UserInfo user = userUtil.getUser();

        QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
        AssertUtil.isNull(qrPackage, "该码包不存在！");

        qrPackage.setInvalid(InvalidEnum.INVALID.getCode());
        qrPackage.setUpdateBy(user.getId());
        qrPackage.setUpdateTime(LocalDateTime.now());

        qrPackageMapper.updateById(qrPackage);

    }

    @Override
    public void invalidQrSingle(InvalidQrSingleReq req) {


        try {
            //设置主键code 表名 列名称
            String code = "code", tableName = "qr_code", columnName = "invalid";

            //根据主键code更新列字段
            tableStoreUtils.updateRow(code, req.getCode(), tableName, columnName, InvalidEnum.INVALID.getCode().toString());

        } catch (Exception e) {
            log.info("更新tableStore异常", e);
            throw new BusinessException("更新tableStore异常");
        }
    }

    @Override
    public GetPackageCodeRes getPackageCodeType(GetPackageCodeReq req) {
        //1-查询sap订单明细
        QrPackageSapOrderItem orderItem = qrPackageSapOrderItemMapper.selectById(req.getOrderItemId());
        AssertUtil.isNull(orderItem, "订单明细不存在");

        //2-查询对应物料
        BaseMaterial material = materialService.getMaterialByCode(orderItem.getSapMaterialId());
        AssertUtil.isNull(material, String.format("SAP物料%s未创建对应的QMS系统内物料！", orderItem.getSapMaterialId()));

        //3-查询包材厂
        BaseFactory coverFactory = factoryService.getFactoryByCode(orderItem.getSapCoverFactoryId(), FactoryTypeEnum.COVER.getCode());
        AssertUtil.isNull(coverFactory, String.format("SAP包材厂%s未创建对应的QMS系统内包材厂！", orderItem.getSapCoverFactoryId()));

        //4-查询啤酒厂
        BaseFactory beerFactory = factoryService.getFactoryByCode(orderItem.getSapBeerFactoryId(), FactoryTypeEnum.BEER.getCode());
        AssertUtil.isNull(beerFactory, String.format("SAP啤酒厂%s未创建对应的QMS系统内啤酒厂！", orderItem.getSapBeerFactoryId()));

        GetPackageCodeRes res = new GetPackageCodeRes();
        res.setFactoryCoverId(coverFactory.getId());
        res.setFactoryBeerId(beerFactory.getId());
        res.setFactoryCoverName(coverFactory.getFactoryName());
        res.setFactoryBeerName(beerFactory.getFactoryName());
        res.setIsFirstFactory(coverFactory.getIsFirstFactory());
        res.setMaterialId(material.getId());
        res.setPurchaseCount(orderItem.getPurchaseCount());
        res.setQrType(getPackageCode(material.getMaterialType()));

        return res;
    }

    @Override
    public ListQrPackageWxRes listWxQrPackage(ListQrPackageWxReq listQrPackageWxReq) {

        // 是否小程序用户主账号 1-是 0-不是
        Integer isMainAccount = userUtil.getWholeUser().getIsMainAccount();

        ListQrPackageWxRes res = new ListQrPackageWxRes();

        List<QrPackageWxVO> qrPackageWxVOList = qrPackageMapper.listWxQrPackage(listQrPackageWxReq);

        List<ListQrPackageWxRes.QrPackageWxBean> beanList = PojoConverterUtils.copyList(qrPackageWxVOList, ListQrPackageWxRes.QrPackageWxBean.class);
        res.setQrPackageWxBeanList(beanList);
        res.setIsMainAccount(isMainAccount);

        return res;
    }

    @Override
    public WxQrPackageDetailRes getWxQrPackageDetail(WxQrPackageDetailReq req) {

        //详情
        WxQrPackageDetailVO detail = qrPackageMapper.getWxQrPackageDetail(req);
        WxQrPackageDetailRes res = PojoConverterUtils.copy(detail, WxQrPackageDetailRes.class);

        //箱码列表
        QueryWrapper<QrBoxMapping> qw = new QueryWrapper<>();
        qw.eq("package_id", req.getPackageId());
        qw.eq("is_delete", 0);
        List<QrBoxMapping> boxMappingList = qrBoxMappingMapper.selectList(qw);
        List<WxQrPackageDetailRes.QrBoxMappingBean> beanList = PojoConverterUtils.copyList(boxMappingList, WxQrPackageDetailRes.QrBoxMappingBean.class);
        res.setQrBoxMappingBeanList(beanList);

        return res;
    }

    /**
     * 根据物料类型转换码类型
     * 物料类型： 3-罐盖 4-瓶盖 5-纸板箱
     * 码类型：1-箱码 2-盖码
     *
     * @param type
     * @return
     */
    public Integer getPackageCode(Integer type) {
        if (MaterialTypeEnum.CARTON.is(type)) {
            return QrTypeEnum.CARTON.getCode();
        } else if (MaterialTypeEnum.BOTTLE_CAP.is(type) || MaterialTypeEnum.JAR.is(type)) {
            return QrTypeEnum.CAP.getCode();
        } else {
            throw new BusinessException("物料类型和对应的箱码类型不匹配！");
        }

    }
}
