package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.enums.QrTypeEnum;
import com.abi.qms.platform.dao.mapper.ReportServiceMapper;
import com.abi.qms.platform.dao.vo.result.*;
import com.abi.qms.platform.dto.req.QueryWxFactoryReq;
import com.abi.qms.platform.dto.req.ReportReq;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.ReportService;
import com.abi.task.common.utils.PojoConverterUtils;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.DayOfWeek;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.math.BigDecimal;
import java.util.stream.Collectors;

/**
 * @author ludashi
 * @date 2021年06月02日 15:39
 */
@Service
public class ReportServiceImpl implements ReportService {

    @Autowired
    private ReportServiceMapper reportServiceMapper;

    @Autowired
    private UserUtil userUtil;


    /**
     * 品牌使用码量占比（603版 已废弃）
     *
     * @param reportReq
     * @return com.abi.qms.platform.dto.res.ListBrandCodeReportRes
     * @author ludashi
     * @date 2021/6/2 17:50
     */
    /*@Override
    public ListBrandCodeReportRes queryBrandCodeProportion(ReportReq reportReq) {
        //参数设置初始化
        setQueryTime(reportReq);
        //查询 品牌数量
        List<ListBrandCodeReportVO> listReportVOS = reportServiceMapper.queryBrandCodeProportion(reportReq);
        if(!CollectionUtils.isEmpty(listReportVOS)){
            //求总和
            Long collect = listReportVOS.stream().collect(Collectors.summingLong(ListBrandCodeReportVO::getQrNumber));
            listReportVOS.forEach(listReportVO->{
                Double val =listReportVO.getQrNumber().doubleValue() / collect.doubleValue()* 100;
                listReportVO.setPercent(new DecimalFormat("#0.00").format(val + 0.000001d)+"%");
            });
        }
        // 封装出参
        ListBrandCodeReportRes res = new ListBrandCodeReportRes();
        List<ListBrandCodeReportRes.BrandCodeReportBean> reportRes = PojoConverterUtils.copyList(listReportVOS, ListBrandCodeReportRes.BrandCodeReportBean.class);
        res.setReportBrandCodeResList(reportRes);
        return res;
    }*/

    /**
     * 物料使用码量排名
     * @param reportReq
     * @return
     */
    @Override
    public ListMaterialCodeRankingReportRes queryMaterialCodeRanking(ReportReq reportReq) {
        List<ListMaterialCodeReportVO> listMaterialCodeReportVOS = reportServiceMapper.queryMaterialCodeRanking(reportReq);
        // 封装出参
        ListMaterialCodeRankingReportRes res = new ListMaterialCodeRankingReportRes();
        List<ListMaterialCodeRankingReportRes.MaterialCodeReportResBean> materialCodeReportResBeans = PojoConverterUtils.copyList(listMaterialCodeReportVOS, ListMaterialCodeRankingReportRes.MaterialCodeReportResBean.class);
        res.setReportMaterialCodeRankingResList(materialCodeReportResBeans);
        return res;
    }


    /**
     * 包材厂使用码量，下载率
     * @param reportReq
     * @return
     */
    @Override
    public ListFactoryCodeReportRes queryFactoryCodeTotalReport(ReportReq reportReq) {
        //参数设置初始化
        setQueryTime(reportReq);
        //包材厂生成包数量
        List<ListFactoryCodeReportVO> listFactoryCodeTotalReportVOS = reportServiceMapper.queryFactoryCodeTotal(reportReq,0);
        //包材厂包下载量
        List<ListFactoryCodeReportVO> listFactoryCodeDownloadTotalReportVOS = reportServiceMapper.queryFactoryCodeTotal(reportReq,1);
        Map<String, Long> factoryCodeDownloadTotalMaps = listFactoryCodeDownloadTotalReportVOS.stream().collect(Collectors.toMap(ListFactoryCodeReportVO::getFactoryName, ListFactoryCodeReportVO::getQrNumber));
        if(!CollectionUtils.isEmpty(listFactoryCodeTotalReportVOS)) {
            listFactoryCodeTotalReportVOS.forEach(listReportVO -> {
                Double val = 0d;
                //计算下载率
                if (null != factoryCodeDownloadTotalMaps.get(listReportVO.getFactoryName())) {
                    val = factoryCodeDownloadTotalMaps.get(listReportVO.getFactoryName()).doubleValue() / listReportVO.getQrNumber().doubleValue() * 100;
                }
                //下载率取整
                listReportVO.setDownloadNumber(new BigDecimal(Math.floor(val)).longValue());
            });
        }
        // 封装出参
        ListFactoryCodeReportRes res = new ListFactoryCodeReportRes();
        List<ListFactoryCodeReportRes.FactoryCodeReportResBean> reportFactoryRes = PojoConverterUtils.copyList(listFactoryCodeTotalReportVOS, ListFactoryCodeReportRes.FactoryCodeReportResBean.class);
        res.setFactoryCodeResReportList(reportFactoryRes);
        return res;
    }

    /**
     * 统计下单量
     * @author lu
     * @date 2021/6/10 11:35
     * @param queryWxFactoryReq
     * @return com.abi.qms.platform.dto.res.QueryWxFactoryRes
     */
    @Override
    public QueryWxFactoryRes queryWxFactory(QueryWxFactoryReq queryWxFactoryReq) {
        // 是小程序用户主账号 1-是 0-不是
        Integer isMainAccount = userUtil.getWholeUser().getIsMainAccount();
        QueryWxFactoryRes res = new QueryWxFactoryRes();
        if (isMainAccount.equals(0)) {
            //子账号报表
            //获取现在时间 获取前三个月的时间
            List<QueryWxFactoryVO> queryWxFactoryVO = reportServiceMapper.queryWxFactory(queryWxFactoryReq,LocalDateTime.now(),getMonth());
            AssertUtil.isNull(queryWxFactoryVO,"报表数据查询为空");
            List<QueryWxFactoryRes.QueryWxFactoryBean> queryWxFactoryBeans = PojoConverterUtils.copyList(queryWxFactoryVO, QueryWxFactoryRes.QueryWxFactoryBean.class);
            queryWxFactoryBeans.forEach(beans->{
                beans.setInStock(beans.getTotal() - beans.getActivation());
            });
            res.setQueryWxFactoryBeanList(queryWxFactoryBeans);
            return res;
        }
        //下单量-激活量-库存
        List<QueryWxFactoryVO> queryWxFactoryVO = reportServiceMapper.queryWxFactory(queryWxFactoryReq,null,null);
        AssertUtil.isNull(queryWxFactoryVO,"查询为空");
        List<QueryWxFactoryRes.QueryWxFactoryBean> queryWxFactoryBeans = PojoConverterUtils.copyList(queryWxFactoryVO, QueryWxFactoryRes.QueryWxFactoryBean.class);
        Long total = queryWxFactoryBeans.stream().mapToLong(QueryWxFactoryRes.QueryWxFactoryBean::getTotal).sum();
        Long activation = queryWxFactoryBeans.stream().mapToLong(QueryWxFactoryRes.QueryWxFactoryBean::getActivation).sum();
        Long stock= total-activation;
        QueryWxFactoryRes.QueryWxFactoryBean factoryBean= new QueryWxFactoryRes.QueryWxFactoryBean();
        factoryBean.setActivation(activation);
        factoryBean.setInStock(stock);
        factoryBean.setTotal(total);
        List<QueryWxFactoryRes.QueryWxFactoryBean> list = new ArrayList<>();
        list.add(factoryBean);
        res.setQueryWxFactoryBeanList(list);
        return res;

    }

    /**
     * 设置时间区间值
     * @param reportReq
     */
    private void setQueryTime(ReportReq reportReq){
        if(!ObjectUtils.isEmpty(reportReq) &&
                null == reportReq.getEndTime() &&
                null == reportReq.getBeginTime() &&
                !ObjectUtils.isEmpty(reportReq.getValue())){
            HashMap<String, String> timestamp = getTimestamp(reportReq.getValue());
            reportReq.setBeginTime( timestamp.get("startTime"));
            reportReq.setEndTime( timestamp.get("endTime"));
        }
    }

    /**
     * 根据类型获取时间范围
     * @param timeType (0-当天，1-本周，2-本月)
     * @return key 开始时间：startTime  结束时间：endTime
     */
    private HashMap<String, String> getTimestamp(Integer timeType) {
        HashMap<String, String> hashMap = new HashMap<String, String>();
        Calendar calendar = Calendar.getInstance();

        //当天
        if (timeType == 0){
            hashMap.put("startTime", longTurnData(getTimestampByOffsetDay(0)));
            hashMap.put("endTime", longTurnData(getTimestampByOffsetDay(1)));
        }
        //本周
        if (timeType == 1){
            hashMap.put(
                    "startTime",
                    longTurnData(getTimestampByOffsetDay(0 - calendar.get(Calendar.DAY_OF_WEEK) + 2)) );
            hashMap.put(
                    "endTime",
                    longTurnData( getTimestampByOffsetDay(calendar
                            .getMaximum(Calendar.DAY_OF_WEEK)
                            - calendar.get(Calendar.DAY_OF_WEEK) + 2)));
        }

        //本月
        if (timeType == 2){
            hashMap.put(
                    "startTime",
                    longTurnData(getTimestampByOffsetDay(0 - calendar.get(Calendar.DAY_OF_MONTH) + 1)));
            hashMap.put(
                    "endTime",
                    longTurnData( getTimestampByOffsetDay(calendar
                            .getMaximum(Calendar.DAY_OF_MONTH)
                            - calendar.get(Calendar.DAY_OF_MONTH))));
        }
        return hashMap;
    }

    /**
     * 时间格式转化
     * @author ludashi
     * @date 2021/6/3 13:37
     * @param l 时间毫秒值
     * @return String
     */
    private static String longTurnData(Long l) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = new Date(l);
        return simpleDateFormat.format(date);
    }

    /**
     * 时间定义
     * @author ludashi
     * @date 2021/6/3 13:12
     * @param day
     * @return long
     */
    public long getTimestampByOffsetDay(int day){
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DAY_OF_MONTH, day);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTimeInMillis();
    }

    /**
     * 码包概况
     * @author liguosong
     * @date 2021/6/3
     * @param reportReq
     * @return
     */
    @Override
    public PackageCodeTypeRes queryPackageCodeGeneral(ReportReq reportReq) {
        //0-准备时间参数
        Map<String,String> map= fillParam(reportReq);
        PackageCodeTypeRes res = new PackageCodeTypeRes();

        //1-查询本期
        //查询同期生成码(参数：开始时间，结束时间，1-生成码类型/2-下载码类型，1-同期/2-往期)
        ListPackageCodeVo createBean = reportServiceMapper.queryPackageCodeNum(reportReq.getBeginTime(),reportReq.getEndTime(),1);
        res.setCreateBoxNumber(createBean.getQrBoxNumber());
        res.setCreateCapNumber(createBean.getQrCapNumber());
        //查询同期下载码
        ListPackageCodeVo downloadBean = reportServiceMapper.queryPackageCodeNum(reportReq.getBeginTime(),reportReq.getEndTime(),2);
        res.setLoadBoxNumber(downloadBean.getQrBoxNumber());
        res.setLoadCapNumber(downloadBean.getQrCapNumber());
        //查询同期激活码 （参数：开始时间，结束时间，1-同期/2-往期）
        ListPackageCodeVo activateBean = reportServiceMapper.queryActivateCodeNum(reportReq.getBeginTime(),reportReq.getEndTime());
        res.setActivateBoxNumber(activateBean.getQrBoxNumber());
        res.setActivateCapNumber(activateBean.getQrCapNumber());

        //2-查询往期并计算同比增长率
        if(!map.isEmpty()){
            //查询往期生成码
            ListPackageCodeVo createPresentBean = reportServiceMapper.queryPackageCodeNum(map.get("beginDate"), map.get("endDate"), 1);
            //查询往期下载码（参数：开始时间，结束时间，1-生成码类型/2-下载码类型，1-同期/2-往期）
            ListPackageCodeVo downloadPresentBean = reportServiceMapper.queryPackageCodeNum(map.get("beginDate"), map.get("endDate"), 2);
            //查询往期激活码（参数：开始时间，结束时间，1-同期/2-往期）
            ListPackageCodeVo activatePresentBean = reportServiceMapper.queryActivateCodeNum(map.get("beginDate"), map.get("endDate"));
            //计算增长率
            res.setCreatePercent(percentBigDecimal(res.getCreateBoxNumber()+res.getCreateCapNumber(), createPresentBean.getQrBoxNumber()+createPresentBean.getQrCapNumber()));
            res.setLoadPercent(percentBigDecimal(res.getLoadBoxNumber()+res.getLoadCapNumber(), downloadPresentBean.getQrBoxNumber()+downloadPresentBean.getQrCapNumber()));
            res.setActivatePercent(percentBigDecimal(res.getActivateBoxNumber()+res.getActivateCapNumber(), activatePresentBean.getQrBoxNumber()+activatePresentBean.getQrCapNumber()));
        }

        //返回参数
        return res;
    }



    /**
     * 封装入参
     * @param reportReq
     */
    private Map<String,String> fillParam(ReportReq reportReq) {
        if (ObjectUtils.isEmpty(reportReq) || ObjectUtils.isEmpty(reportReq.getValue())){
            return Maps.newHashMap();
        }
        setQueryTime(reportReq);
        return getBeforeWeekTime(reportReq.getValue().toString());
    }

    /**
     * @param present 同期数
     * @param before 往期数
     * @return
     */
    public static double percentBigDecimal(long present, long before) {
        BigDecimal preNum = BigDecimal.valueOf(present);
        BigDecimal sufNum = BigDecimal.valueOf(before);
        double result = countDecimal(preNum, sufNum);
        if (result > 0) {
            return  result;
        }
        if (result < 0) {
            return -Math.abs(result);
        }
        if (result == 0) {
            return 0;
        }
        return 0;
    }

    public static double countDecimal(BigDecimal preNum, BigDecimal sufNum) {
        boolean preBoolean = verifyNum(preNum);
        boolean sufBoolean = verifyNum(sufNum);
        //同时为true计算
        if (preBoolean && sufBoolean) {
            boolean b = verifyEqual(preNum, sufNum);
            if (b == false) {
                return realCountDecimal(preNum, sufNum);
            }
            if (b) {
                return 0;
            }
        }
        if (preBoolean == false && sufBoolean == false) {
            return 0;
        }
        if (sufBoolean == false) {
            return 100;
        }
        if (preBoolean == false) {
            return -100;
        }
        return 0;
    }

    //验证数字是否为零和null
    public static boolean verifyNum(BigDecimal num) {
        if (null != num && num.compareTo(BigDecimal.ZERO) != 0) {
            return true;
        }
        return false;
    }

    //验证两个数字是否相等
    public static boolean verifyEqual(BigDecimal preNum, BigDecimal sufNum) {
        int n = preNum.compareTo(sufNum);
        if (n == 0) {
            return true;
        }
        return false;
    }

    //真正计算
    public static double realCountDecimal(BigDecimal preNum, BigDecimal sufNum) {
        //(前面的数字-后面的数字)/后面的数字*100
        BigDecimal bigDecimal = (preNum.subtract(sufNum)).divide(sufNum, 2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("100")).setScale(2, BigDecimal.ROUND_HALF_UP);
        if (bigDecimal.compareTo(BigDecimal.ZERO) != 0) {
            return bigDecimal.doubleValue();
        }
        return 0;
    }

    /**
     * 获取往期时间（1-周，2-月）
     * @return
     */
    public static Map<String,String> getBeforeWeekTime(String value){
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd ");
        Map<String,String> map=new HashMap<String,String>();
        Calendar calStart = Calendar.getInstance();
        Calendar calEnd = Calendar.getInstance();
        if("1".equals(value)){
            calStart.add(Calendar.DAY_OF_MONTH,-7);//上一周
            calEnd.add(Calendar.DAY_OF_MONTH,-7);
            calStart.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
            calEnd.setFirstDayOfWeek(Calendar.MONDAY);
            calEnd.set(Calendar.DAY_OF_WEEK, calEnd.getFirstDayOfWeek() + 6);
        }else{
            calStart.add(Calendar.MONTH, -1);//上一月
            calEnd.add(Calendar.MONTH, -1);
            calStart.set(Calendar.DAY_OF_MONTH, 1);
            calEnd.set(Calendar.DAY_OF_MONTH, calEnd.getActualMaximum(Calendar.DAY_OF_MONTH));
        }
        String beginTime = simpleDateFormat.format(calStart.getTime()) + "00:00:00";
        String endTime = simpleDateFormat.format(calEnd.getTime()) + "23:59:59";
        map.put("beginDate",beginTime);
        map.put("endDate",endTime);
        return map;
    }

    /**
     * 获取进三个月的
     * @author lu
     * @date 2021/6/10 17:03
     * @return java.time.LocalDateTime
     */
    private LocalDateTime getMonth(){
        Calendar calStart = Calendar.getInstance();
        calStart.add(Calendar.MONTH, -3);//上一月
        calStart.set(Calendar.DAY_OF_MONTH, 1);
        calStart.set(Calendar.HOUR_OF_DAY, 0);
        calStart.set(Calendar.SECOND, 0);
        calStart.set(Calendar.MINUTE, 0);
        calStart.set(Calendar.MILLISECOND, 0);
        Instant instant = calStart.toInstant();
        ZoneId zoneId = ZoneId.systemDefault();
        return instant.atZone(zoneId).toLocalDateTime();
    }
}